# This file is part of Grow-bots.
#
# Grow-bots is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Grow-bots is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Grow-bots.  If not, see <http://www.gnu.org/licenses/>.

import pygame
from OpenGL.GL import *
from OpenGL.GLU import *

from _keys import Keys
from _mouse import Mouse
from _events import Events
from _gui import Gui

"""
I couldn't think of a name for this module, so I called it "Dyna"
after my cat :) It's just a simple "round-up" of all the main-loop
related objects / routines.
"""

class Dyna:

    context = None

    def __init__(self, width, height, fps):
        self.width, self.height = width, height
        self.fps = fps
        pygame.init()
        flags = pygame.OPENGL | pygame.DOUBLEBUF
        pygame.display.set_mode((self.width, self.height), flags)
        self.events = Events()
        self.events.register("keydown")
        self.events.register("keyup")
        self.events.register("buttondown")
        self.events.register("buttonup")
        self.events.register("mousemotion")
        self.events.register("start")
        self.events.register("stop")
        self.events.register("update")
        self.events.register("render")
        self.events.register("process")
        self.keys = Keys(self.events)
        self.mouse = Mouse(self.events)
        self.clock = pygame.time.Clock()
        self.init_opengl()

    def push2d(self):
        glPushAttrib(GL_LIGHTING_BIT)
        glDisable(GL_LIGHTING)
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        glOrtho(0, self.width, self.height, 0, -1, 1)
        glMatrixMode(GL_MODELVIEW)
        glPushMatrix()
        glLoadIdentity()
        glDisable(GL_DEPTH_TEST)

    def pop2d(self):
        glPopAttrib(GL_LIGHTING_BIT)
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glMatrixMode(GL_MODELVIEW)
        glPopMatrix()
        glEnable(GL_DEPTH_TEST)

    def init_opengl(self):
        glViewport(0, 0, self.width, self.height)
        glMatrixMode(GL_PROJECTION)
        gluPerspective(60, float(self.width) / self.height, 0.1, 1000)
        glMatrixMode(GL_MODELVIEW)
        glEnable(GL_DEPTH_TEST)
        glAlphaFunc(GL_GREATER, 0.3)

    def render(self, flip=True):
        glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT)
        glLoadIdentity()
        self.events.send("render")
        if flip:
            pygame.display.flip()

    def update(self):
        self.process()
        self.clock.tick(self.fps)
        self.events.send("update")

    def process(self):
        for e in pygame.event.get():
            if e.type == pygame.QUIT:
                self.running = False
            elif e.type == pygame.KEYDOWN:
                self.events.send("keydown", e.key)
            elif e.type == pygame.KEYUP:
                self.events.send("keyup", e.key)
            elif e.type == pygame.MOUSEBUTTONDOWN:
                self.events.send("buttondown", e.button)
            elif e.type == pygame.MOUSEBUTTONUP:
                self.events.send("buttonup", e.button)
            elif e.type == pygame.MOUSEMOTION:
                self.events.send("mousemotion", e.pos)
        self.events.send("process")

    def step(self):
        self.render()
        self.update()

    def start(self):
        self.running = True
        self.events.send("start")
        while self.running:
            self.step()
        self.events.send("stop")
        pygame.quit()

    def stop(self):
        self.running = False


def init(width=640, height=480, fps=40):
    if not Dyna.context:
        Dyna.context = Dyna(width, height, fps)
    else:
        raise Exception("Dyna can only be initialized once!")

def getcontext():
    if not Dyna.context:
        raise Exception("Dyna context has not been initialized!")
    return Dyna.context
