# This file is part of Grow-bots.
#
# Grow-bots is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Grow-bots is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Grow-bots.  If not, see <http://www.gnu.org/licenses/>.

import dyna

"""
Manages keypress events
"""

class Keys:

    def __init__(self, events):
        self.key = {}
        self.parent = events
        self.events = dyna.Events()
        events.add("keydown", self.set_down)
        events.add("keyup", self.set_up)

    def add_keydown(self, key, func):
        self.events.add("keydown_%i" % int(key), func)

    def add_keyup(self, key, func):
        self.events.add("keyup_%i" % int(key), func)

    def remove_keydown(self, key, func):
        self.events.remove("keydown_%i" % int(key), func)

    def remove_keyup(self, key, func):
        self.events.remove("keyup_%i" % int(key), func)

    def set_down(self, data):
        self.key[data] = True
        self.events.send("keydown_%i" % int(data), data)

    def set_up(self, data):
        self.key[data] = False
        self.events.send("keyup_%i" % int(data), data)

    def is_down(self, key):
        if key in self.key:
            return self.key[key]
        else:
            return False

    def on_down(self, key):
        if key in self.key:
            state = self.key[key]
            self.key[key] = False
            return state
        else:
            return False

    def clear(self):
        for key in self.key:
            self.key[key] = False
